# -*- coding: utf-8 -*-

"""
:maintainer:    Alberto Planas <aplanas@suse.com>
:platform:      Linux
"""
# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import pytest
import salt.states.btrfs as btrfs
import salt.utils.platform
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf


@skipIf(salt.utils.platform.is_windows(), "Non-Windows feature")
class BtrfsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.btrfs
    """

    def setup_loader_modules(self):
        return {btrfs: {"__salt__": {}, "__states__": {}, "__utils__": {}}}

    @patch("salt.states.btrfs._umount")
    @patch("tempfile.mkdtemp")
    def test__mount_fails(self, mkdtemp, umount):
        """
        Test mounting a device in a temporary place.
        """
        mkdtemp.return_value = "/tmp/xxx"
        states_mock = {
            "mount.mounted": MagicMock(return_value={"result": False}),
        }
        with patch.dict(btrfs.__states__, states_mock):
            assert btrfs._mount("/dev/sda1", use_default=False) is None
            mkdtemp.assert_called_once()
            states_mock["mount.mounted"].assert_called_with(
                "/tmp/xxx",
                device="/dev/sda1",
                fstype="btrfs",
                opts="subvol=/",
                persist=False,
            )
            umount.assert_called_with("/tmp/xxx")

    @patch("salt.states.btrfs._umount")
    @patch("tempfile.mkdtemp")
    def test__mount(self, mkdtemp, umount):
        """
        Test mounting a device in a temporary place.
        """
        mkdtemp.return_value = "/tmp/xxx"
        states_mock = {
            "mount.mounted": MagicMock(return_value={"result": True}),
        }
        with patch.dict(btrfs.__states__, states_mock):
            assert btrfs._mount("/dev/sda1", use_default=False) == "/tmp/xxx"
            mkdtemp.assert_called_once()
            states_mock["mount.mounted"].assert_called_with(
                "/tmp/xxx",
                device="/dev/sda1",
                fstype="btrfs",
                opts="subvol=/",
                persist=False,
            )
            umount.assert_not_called()

    @patch("salt.states.btrfs._umount")
    @patch("tempfile.mkdtemp")
    def test__mount_use_default(self, mkdtemp, umount):
        """
        Test mounting a device in a temporary place.
        """
        mkdtemp.return_value = "/tmp/xxx"
        states_mock = {
            "mount.mounted": MagicMock(return_value={"result": True}),
        }
        with patch.dict(btrfs.__states__, states_mock):
            assert btrfs._mount("/dev/sda1", use_default=True) == "/tmp/xxx"
            mkdtemp.assert_called_once()
            states_mock["mount.mounted"].assert_called_with(
                "/tmp/xxx",
                device="/dev/sda1",
                fstype="btrfs",
                opts="defaults",
                persist=False,
            )
            umount.assert_not_called()

    def test__umount(self):
        """
        Test umounting and cleanning temporary place.
        """
        states_mock = {
            "mount.unmounted": MagicMock(),
        }
        utils_mock = {
            "files.rm_rf": MagicMock(),
        }
        with patch.dict(btrfs.__states__, states_mock), patch.dict(
            btrfs.__utils__, utils_mock
        ):
            btrfs._umount("/tmp/xxx")
            states_mock["mount.unmounted"].assert_called_with("/tmp/xxx")
            utils_mock["files.rm_rf"].assert_called_with("/tmp/xxx")

    def test__is_default_not_default(self):
        """
        Test if the subvolume is the current default.
        """
        salt_mock = {
            "btrfs.subvolume_show": MagicMock(
                return_value={"@/var": {"subvolume id": "256"}}
            ),
            "btrfs.subvolume_get_default": MagicMock(return_value={"id": "5"}),
        }
        with patch.dict(btrfs.__salt__, salt_mock):
            assert not btrfs._is_default("/tmp/xxx/@/var", "/tmp/xxx", "@/var")
            salt_mock["btrfs.subvolume_show"].assert_called_with("/tmp/xxx/@/var")
            salt_mock["btrfs.subvolume_get_default"].assert_called_with("/tmp/xxx")

    def test__is_default(self):
        """
        Test if the subvolume is the current default.
        """
        salt_mock = {
            "btrfs.subvolume_show": MagicMock(
                return_value={"@/var": {"subvolume id": "256"}}
            ),
            "btrfs.subvolume_get_default": MagicMock(return_value={"id": "256"}),
        }
        with patch.dict(btrfs.__salt__, salt_mock):
            assert btrfs._is_default("/tmp/xxx/@/var", "/tmp/xxx", "@/var")
            salt_mock["btrfs.subvolume_show"].assert_called_with("/tmp/xxx/@/var")
            salt_mock["btrfs.subvolume_get_default"].assert_called_with("/tmp/xxx")

    def test__set_default(self):
        """
        Test setting a subvolume as the current default.
        """
        salt_mock = {
            "btrfs.subvolume_show": MagicMock(
                return_value={"@/var": {"subvolume id": "256"}}
            ),
            "btrfs.subvolume_set_default": MagicMock(return_value=True),
        }
        with patch.dict(btrfs.__salt__, salt_mock):
            assert btrfs._set_default("/tmp/xxx/@/var", "/tmp/xxx", "@/var")
            salt_mock["btrfs.subvolume_show"].assert_called_with("/tmp/xxx/@/var")
            salt_mock["btrfs.subvolume_set_default"].assert_called_with(
                "256", "/tmp/xxx"
            )

    def test__is_cow_not_cow(self):
        """
        Test if the subvolume is copy on write.
        """
        salt_mock = {
            "file.lsattr": MagicMock(return_value={"/tmp/xxx/@/var": ["C"]}),
        }
        with patch.dict(btrfs.__salt__, salt_mock):
            assert not btrfs._is_cow("/tmp/xxx/@/var")
            salt_mock["file.lsattr"].assert_called_with("/tmp/xxx/@")

    def test__is_cow(self):
        """
        Test if the subvolume is copy on write.
        """
        salt_mock = {
            "file.lsattr": MagicMock(return_value={"/tmp/xxx/@/var": []}),
        }
        with patch.dict(btrfs.__salt__, salt_mock):
            assert btrfs._is_cow("/tmp/xxx/@/var")
            salt_mock["file.lsattr"].assert_called_with("/tmp/xxx/@")

    def test__unset_cow(self):
        """
        Test disabling the subvolume as copy on write.
        """
        salt_mock = {
            "file.chattr": MagicMock(return_value=True),
        }
        with patch.dict(btrfs.__salt__, salt_mock):
            assert btrfs._unset_cow("/tmp/xxx/@/var")
            salt_mock["file.chattr"].assert_called_with(
                "/tmp/xxx/@/var", operator="add", attributes="C"
            )

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created_exists(self, mount, umount):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=True),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.subvolume_created(name="@/var", device="/dev/sda1") == {
                "name": "@/var",
                "result": True,
                "changes": {},
                "comment": ["Subvolume @/var already present"],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created_exists_test(self, mount, umount):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=True),
        }
        opts_mock = {
            "test": True,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.subvolume_created(name="@/var", device="/dev/sda1") == {
                "name": "@/var",
                "result": None,
                "changes": {},
                "comment": ["Subvolume @/var already present"],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._is_default")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created_exists_was_default(self, mount, umount, is_default):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        is_default.return_value = True
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=True),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.subvolume_created(
                name="@/var", device="/dev/sda1", set_default=True
            ) == {
                "name": "@/var",
                "result": True,
                "changes": {},
                "comment": ["Subvolume @/var already present"],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._set_default")
    @patch("salt.states.btrfs._is_default")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created_exists_set_default(
        self, mount, umount, is_default, set_default
    ):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        is_default.return_value = False
        set_default.return_value = True
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=True),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.subvolume_created(
                name="@/var", device="/dev/sda1", set_default=True
            ) == {
                "name": "@/var",
                "result": True,
                "changes": {"@/var_default": True},
                "comment": ["Subvolume @/var already present"],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._set_default")
    @patch("salt.states.btrfs._is_default")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created_exists_set_default_no_force(
        self, mount, umount, is_default, set_default
    ):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        is_default.return_value = False
        set_default.return_value = True
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=True),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.subvolume_created(
                name="@/var",
                device="/dev/sda1",
                set_default=True,
                force_set_default=False,
            ) == {
                "name": "@/var",
                "result": True,
                "changes": {},
                "comment": ["Subvolume @/var already present"],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._is_cow")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created_exists_no_cow(self, mount, umount, is_cow):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        is_cow.return_value = False
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=True),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.subvolume_created(
                name="@/var", device="/dev/sda1", copy_on_write=False
            ) == {
                "name": "@/var",
                "result": True,
                "changes": {},
                "comment": ["Subvolume @/var already present"],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._unset_cow")
    @patch("salt.states.btrfs._is_cow")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created_exists_unset_cow(self, mount, umount, is_cow, unset_cow):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        is_cow.return_value = True
        unset_cow.return_value = True
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=True),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.subvolume_created(
                name="@/var", device="/dev/sda1", copy_on_write=False
            ) == {
                "name": "@/var",
                "result": True,
                "changes": {"@/var_no_cow": True},
                "comment": ["Subvolume @/var already present"],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created(self, mount, umount):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=False),
            "btrfs.subvolume_create": MagicMock(),
        }
        states_mock = {
            "file.directory": MagicMock(return_value={"result": True}),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__states__, states_mock
        ), patch.dict(btrfs.__opts__, opts_mock):
            assert btrfs.subvolume_created(name="@/var", device="/dev/sda1") == {
                "name": "@/var",
                "result": True,
                "changes": {"@/var": "Created subvolume @/var"},
                "comment": [],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            salt_mock["btrfs.subvolume_create"].assert_called_once()
            mount.assert_called_once()
            umount.assert_called_once()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created_fails_directory(self, mount, umount):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=False),
        }
        states_mock = {
            "file.directory": MagicMock(return_value={"result": False}),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__states__, states_mock
        ), patch.dict(btrfs.__opts__, opts_mock):
            assert btrfs.subvolume_created(name="@/var", device="/dev/sda1") == {
                "name": "@/var",
                "result": False,
                "changes": {},
                "comment": ["Error creating /tmp/xxx/@ directory"],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    def test_subvolume_created_fails(self, mount, umount):
        """
        Test creating a subvolume.
        """
        mount.return_value = "/tmp/xxx"
        salt_mock = {
            "btrfs.subvolume_exists": MagicMock(return_value=False),
            "btrfs.subvolume_create": MagicMock(side_effect=CommandExecutionError),
        }
        states_mock = {
            "file.directory": MagicMock(return_value={"result": True}),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__states__, states_mock
        ), patch.dict(btrfs.__opts__, opts_mock):
            assert btrfs.subvolume_created(name="@/var", device="/dev/sda1") == {
                "name": "@/var",
                "result": False,
                "changes": {},
                "comment": ["Error creating subvolume @/var"],
            }
            salt_mock["btrfs.subvolume_exists"].assert_called_with("/tmp/xxx/@/var")
            salt_mock["btrfs.subvolume_create"].assert_called_once()
            mount.assert_called_once()
            umount.assert_called_once()

    def test_diff_properties_fails(self):
        """
        Test when diff_properties do not found a property
        """
        expected = {"wrong_property": True}
        current = {
            "compression": {
                "description": "Set/get compression for a file or directory",
                "value": "N/A",
            },
            "label": {"description": "Set/get label of device.", "value": "N/A"},
            "ro": {
                "description": "Set/get read-only flag or subvolume",
                "value": "N/A",
            },
        }
        with pytest.raises(Exception):
            btrfs._diff_properties(expected, current)

    def test_diff_properties_enable_ro(self):
        """
        Test when diff_properties enable one single property
        """
        expected = {"ro": True}
        current = {
            "compression": {
                "description": "Set/get compression for a file or directory",
                "value": "N/A",
            },
            "label": {"description": "Set/get label of device.", "value": "N/A"},
            "ro": {
                "description": "Set/get read-only flag or subvolume",
                "value": "N/A",
            },
        }
        assert btrfs._diff_properties(expected, current) == {"ro": True}

    def test_diff_properties_only_enable_ro(self):
        """
        Test when diff_properties is half ready
        """
        expected = {"ro": True, "label": "mylabel"}
        current = {
            "compression": {
                "description": "Set/get compression for a file or directory",
                "value": "N/A",
            },
            "label": {"description": "Set/get label of device.", "value": "mylabel"},
            "ro": {
                "description": "Set/get read-only flag or subvolume",
                "value": "N/A",
            },
        }
        assert btrfs._diff_properties(expected, current) == {"ro": True}

    def test_diff_properties_disable_ro(self):
        """
        Test when diff_properties enable one single property
        """
        expected = {"ro": False}
        current = {
            "compression": {
                "description": "Set/get compression for a file or directory",
                "value": "N/A",
            },
            "label": {"description": "Set/get label of device.", "value": "N/A"},
            "ro": {
                "description": "Set/get read-only flag or subvolume",
                "value": True,
            },
        }
        assert btrfs._diff_properties(expected, current) == {"ro": False}

    def test_diff_properties_emty_na(self):
        """
        Test when diff_properties is already disabled as N/A
        """
        expected = {"ro": False}
        current = {
            "compression": {
                "description": "Set/get compression for a file or directory",
                "value": "N/A",
            },
            "label": {"description": "Set/get label of device.", "value": "N/A"},
            "ro": {
                "description": "Set/get read-only flag or subvolume",
                "value": "N/A",
            },
        }
        assert btrfs._diff_properties(expected, current) == {}

    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    @patch("os.path.exists")
    def test_properties_subvolume_not_exists(self, exists, mount, umount):
        """
        Test when subvolume is not present
        """
        exists.return_value = False
        mount.return_value = "/tmp/xxx"
        assert btrfs.properties(name="@/var", device="/dev/sda1") == {
            "name": "@/var",
            "result": False,
            "changes": {},
            "comment": ["Object @/var not found"],
        }
        mount.assert_called_once()
        umount.assert_called_once()

    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    @patch("os.path.exists")
    def test_properties_default_root_subvolume(self, exists, mount, umount):
        """
        Test when root subvolume resolves to another subvolume
        """
        exists.return_value = False
        mount.return_value = "/tmp/xxx"
        assert btrfs.properties(name="/", device="/dev/sda1") == {
            "name": "/",
            "result": False,
            "changes": {},
            "comment": ["Object / not found"],
        }
        exists.assert_called_with("/tmp/xxx/.")

    @patch("os.path.exists")
    def test_properties_device_fail(self, exists):
        """
        Test when we try to set a device that is not pressent
        """
        exists.return_value = False
        assert btrfs.properties(name="/dev/sda1", device=None) == {
            "name": "/dev/sda1",
            "result": False,
            "changes": {},
            "comment": ["Object /dev/sda1 not found"],
        }

    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    @patch("os.path.exists")
    def test_properties_subvolume_fail(self, exists, mount, umount):
        """
        Test setting a wrong property in a subvolume
        """
        exists.return_value = True
        mount.return_value = "/tmp/xxx"
        salt_mock = {
            "btrfs.properties": MagicMock(
                side_effect=[
                    {
                        "ro": {
                            "description": "Set/get read-only flag or subvolume",
                            "value": "N/A",
                        },
                    }
                ]
            ),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.properties(
                name="@/var", device="/dev/sda1", wrond_property=True
            ) == {
                "name": "@/var",
                "result": False,
                "changes": {},
                "comment": ["Some property not found in @/var"],
            }
            salt_mock["btrfs.properties"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()

    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    @patch("os.path.exists")
    def test_properties_enable_ro_subvolume(self, exists, mount, umount):
        """
        Test setting a ro property in a subvolume
        """
        exists.return_value = True
        mount.return_value = "/tmp/xxx"
        salt_mock = {
            "btrfs.properties": MagicMock(
                side_effect=[
                    {
                        "ro": {
                            "description": "Set/get read-only flag or subvolume",
                            "value": "N/A",
                        },
                    },
                    None,
                    {
                        "ro": {
                            "description": "Set/get read-only flag or subvolume",
                            "value": "true",
                        },
                    },
                ]
            ),
        }
        opts_mock = {
            "test": False,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.properties(name="@/var", device="/dev/sda1", ro=True) == {
                "name": "@/var",
                "result": True,
                "changes": {"ro": "true"},
                "comment": ["Properties changed in @/var"],
            }
            salt_mock["btrfs.properties"].assert_any_call("/tmp/xxx/@/var")
            salt_mock["btrfs.properties"].assert_any_call(
                "/tmp/xxx/@/var", set="ro=true"
            )
            mount.assert_called_once()
            umount.assert_called_once()

    @patch("salt.states.btrfs._umount")
    @patch("salt.states.btrfs._mount")
    @patch("os.path.exists")
    def test_properties_test(self, exists, mount, umount):
        """
        Test setting a property in test mode.
        """
        exists.return_value = True
        mount.return_value = "/tmp/xxx"
        salt_mock = {
            "btrfs.properties": MagicMock(
                side_effect=[
                    {
                        "ro": {
                            "description": "Set/get read-only flag or subvolume",
                            "value": "N/A",
                        },
                    },
                ]
            ),
        }
        opts_mock = {
            "test": True,
        }
        with patch.dict(btrfs.__salt__, salt_mock), patch.dict(
            btrfs.__opts__, opts_mock
        ):
            assert btrfs.properties(name="@/var", device="/dev/sda1", ro=True) == {
                "name": "@/var",
                "result": None,
                "changes": {"ro": "true"},
                "comment": [],
            }
            salt_mock["btrfs.properties"].assert_called_with("/tmp/xxx/@/var")
            mount.assert_called_once()
            umount.assert_called_once()



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.mysql_grants as mysql_grants

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MysqlGrantsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.mysql_grants
    """

    def setup_loader_modules(self):
        return {mysql_grants: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure that the grant is present with the specified properties.
        """
        name = "frank_exampledb"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[True, False, False, False])
        mock_t = MagicMock(return_value=True)
        mock_str = MagicMock(return_value="salt")
        mock_none = MagicMock(return_value=None)
        with patch.dict(
            mysql_grants.__salt__,
            {"mysql.grant_exists": mock, "mysql.grant_add": mock_t},
        ):
            comt = "Grant None on None to None@localhost is already present"
            ret.update({"comment": comt})
            self.assertDictEqual(mysql_grants.present(name), ret)

            with patch.object(mysql_grants, "_get_mysql_error", mock_str):
                ret.update({"comment": "salt", "result": False})
                self.assertDictEqual(mysql_grants.present(name), ret)

            with patch.object(mysql_grants, "_get_mysql_error", mock_none):
                with patch.dict(mysql_grants.__opts__, {"test": True}):
                    comt = "MySQL grant frank_exampledb is set to be created"
                    ret.update({"comment": comt, "result": None})
                    self.assertDictEqual(mysql_grants.present(name), ret)

                with patch.dict(mysql_grants.__opts__, {"test": False}):
                    comt = "Grant None on None to None@localhost" " has been added"
                    ret.update(
                        {"comment": comt, "result": True, "changes": {name: "Present"}}
                    )
                    self.assertDictEqual(mysql_grants.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure that the grant is absent.
        """
        name = "frank_exampledb"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[True, False])
        mock_t = MagicMock(side_effect=[True, True, True, False, False])
        mock_str = MagicMock(return_value="salt")
        mock_none = MagicMock(return_value=None)
        with patch.dict(
            mysql_grants.__salt__,
            {"mysql.grant_exists": mock_t, "mysql.grant_revoke": mock},
        ):
            with patch.dict(mysql_grants.__opts__, {"test": True}):
                comt = "MySQL grant frank_exampledb is set to be revoked"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(mysql_grants.absent(name), ret)

            with patch.dict(mysql_grants.__opts__, {"test": False}):
                comt = "Grant None on None for None@localhost" " has been revoked"
                ret.update(
                    {"comment": comt, "result": True, "changes": {name: "Absent"}}
                )
                self.assertDictEqual(mysql_grants.absent(name), ret)

                with patch.object(mysql_grants, "_get_mysql_error", mock_str):
                    comt = (
                        "Unable to revoke grant None on None"
                        " for None@localhost (salt)"
                    )
                    ret.update({"comment": comt, "result": False, "changes": {}})
                    self.assertDictEqual(mysql_grants.absent(name), ret)

                    comt = (
                        "Unable to determine if grant None on "
                        "None for None@localhost exists (salt)"
                    )
                    ret.update({"comment": comt})
                    self.assertDictEqual(mysql_grants.absent(name), ret)

            with patch.object(mysql_grants, "_get_mysql_error", mock_none):
                comt = (
                    "Grant None on None to None@localhost is not present,"
                    " so it cannot be revoked"
                )
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(mysql_grants.absent(name), ret)



# coding: utf-8

# Python libs
from __future__ import absolute_import

from collections import namedtuple

# Salt libs
import salt.beacons.salt_proxy as salt_proxy
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Salt testing libs
from tests.support.unit import TestCase

PATCH_OPTS = dict(autospec=True, spec_set=True)

FakeProcess = namedtuple("Process", "cmdline pid")


class SaltProxyBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.[s]
    """

    def setup_loader_modules(self):
        return {salt_proxy: {"__context__": {}, "__salt__": {}}}

    def test_non_list_config(self):
        config = {}

        ret = salt_proxy.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for salt_proxy beacon must be a list.")
        )

    def test_empty_config(self):
        config = [{}]

        ret = salt_proxy.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for salt_proxy beacon requires proxies.")
        )

    def test_salt_proxy_running(self):
        mock = MagicMock(return_value={"result": True})
        with patch.dict(salt_proxy.__salt__, {"salt_proxy.is_running": mock}):
            config = [{"proxies": {"p8000": ""}}]

            ret = salt_proxy.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = salt_proxy.beacon(config)
            self.assertEqual(ret, [{"p8000": "Proxy p8000 is already running"}])

    def test_salt_proxy_not_running(self):
        is_running_mock = MagicMock(return_value={"result": False})
        configure_mock = MagicMock(
            return_value={
                "result": True,
                "changes": {
                    "new": "Salt Proxy: Started proxy process for p8000",
                    "old": [],
                },
            }
        )
        cmd_run_mock = MagicMock(
            return_value={"pid": 1000, "retcode": 0, "stderr": "", "stdout": ""}
        )
        with patch.dict(
            salt_proxy.__salt__, {"salt_proxy.is_running": is_running_mock}
        ), patch.dict(
            salt_proxy.__salt__, {"salt_proxy.configure_proxy": configure_mock}
        ), patch.dict(
            salt_proxy.__salt__, {"cmd.run_all": cmd_run_mock}
        ):
            config = [{"proxies": {"p8000": ""}}]

            ret = salt_proxy.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = salt_proxy.beacon(config)
            self.assertEqual(ret, [{"p8000": "Proxy p8000 was started"}])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.keystone as keystone

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MockEC2(object):
    """
    Mock of EC2 class
    """

    def __init__(self):
        self.access = ""
        self.secret = ""
        self.tenant_id = ""
        self.user_id = ""
        self.connection_args = ""
        self.profile = ""

    @staticmethod
    def create(userid, tenantid):
        """
        Mock of create method
        """
        cr_ec2 = MockEC2()
        cr_ec2.tenant_id = tenantid
        cr_ec2.user_id = userid
        return cr_ec2

    def delete(self, userid, accesskey):
        """
        Mock of delete method
        """
        self.access = accesskey
        self.user_id = userid
        return True

    @staticmethod
    def get(user_id, access, profile, **connection_args):
        """
        Mock of get method
        """
        cr_ec2 = MockEC2()
        cr_ec2.profile = profile
        cr_ec2.access = access
        cr_ec2.user_id = user_id
        cr_ec2.connection_args = connection_args
        return cr_ec2

    @staticmethod
    def list(user_id):
        """
        Mock of list method
        """
        cr_ec2 = MockEC2()
        cr_ec2.user_id = user_id
        return [cr_ec2]


class MockEndpoints(object):
    """
    Mock of Endpoints class
    """

    def __init__(self):
        self.id = "007"
        self.region = "RegionOne"
        self.adminurl = "adminurl"
        self.internalurl = "internalurl"
        self.publicurl = "publicurl"
        self.service_id = "117"

    @staticmethod
    def list():
        """
        Mock of list method
        """
        return [MockEndpoints()]

    @staticmethod
    def create(region, service_id, publicurl, adminurl, internalurl):
        """
        Mock of create method
        """
        return (region, service_id, publicurl, adminurl, internalurl)

    @staticmethod
    def delete(id):
        """
        Mock of delete method
        """
        return id


class MockServices(object):
    """
    Mock of Services class
    """

    flag = None

    def __init__(self):
        self.id = "117"
        self.name = "iptables"
        self.description = "description"
        self.type = "type"

    @staticmethod
    def create(name, service_type, description):
        """
        Mock of create method
        """
        service = MockServices()
        service.id = "005"
        service.name = name
        service.description = description
        service.type = service_type
        return service

    def get(self, service_id):
        """
        Mock of get method
        """
        service = MockServices()
        if self.flag == 1:
            service.id = "asd"
            return [service]
        elif self.flag == 2:
            service.id = service_id
            return service
        return [service]

    def list(self):
        """
        Mock of list method
        """
        service = MockServices()
        if self.flag == 1:
            service.id = "asd"
            return [service]
        return [service]

    @staticmethod
    def delete(service_id):
        """
        Mock of delete method
        """
        return service_id


class MockRoles(object):
    """
    Mock of Roles class
    """

    flag = None

    def __init__(self):
        self.id = "113"
        self.name = "nova"
        self.user_id = "446"
        self.tenant_id = "a1a1"

    @staticmethod
    def create(name):
        """
        Mock of create method
        """
        return name

    def get(self, role_id):
        """
        Mock of get method
        """
        role = MockRoles()
        if self.flag == 1:
            role.id = None
            return role
        role.id = role_id
        return role

    @staticmethod
    def list():
        """
        Mock of list method
        """
        return [MockRoles()]

    @staticmethod
    def delete(role):
        """
        Mock of delete method
        """
        return role

    @staticmethod
    def add_user_role(user_id, role_id, tenant_id):
        """
        Mock of add_user_role method
        """
        return (user_id, role_id, tenant_id)

    @staticmethod
    def remove_user_role(user_id, role_id, tenant_id):
        """
        Mock of remove_user_role method
        """
        return (user_id, role_id, tenant_id)

    @staticmethod
    def roles_for_user(user, tenant):
        """
        Mock of roles_for_user method
        """
        role = MockRoles()
        role.user_id = user
        role.tenant_id = tenant
        return [role]


class MockTenants(object):
    """
    Mock of Tenants class
    """

    flag = None

    def __init__(self):
        self.id = "446"
        self.name = "nova"
        self.description = "description"
        self.enabled = "True"

    @staticmethod
    def create(name, description, enabled):
        """
        Mock of create method
        """
        tenant = MockTenants()
        tenant.name = name
        tenant.description = description
        tenant.enabled = enabled
        return tenant

    def get(self, tenant_id):
        """
        Mock of get method
        """
        tenant = MockTenants()
        if self.flag == 1:
            tenant.id = None
            return tenant
        tenant.id = tenant_id
        return tenant

    @staticmethod
    def list():
        """
        Mock of list method
        """
        return [MockTenants()]

    @staticmethod
    def delete(tenant_id):
        """
        Mock of delete method
        """
        return tenant_id


class MockServiceCatalog(object):
    """
    Mock of ServiceCatalog class
    """

    def __init__(self):
        self.id = "446"
        self.expires = "No"
        self.user_id = "admin"
        self.tenant_id = "ae04"

    def get_token(self):
        """
        Mock of get_token method
        """
        return {
            "id": self.id,
            "expires": self.expires,
            "user_id": self.user_id,
            "tenant_id": self.tenant_id,
        }


class MockUsers(object):
    """
    Mock of Users class
    """

    flag = None

    def __init__(self):
        self.id = "446"
        self.name = "nova"
        self.email = "salt@saltstack.com"
        self.enabled = "True"
        self.tenant_id = "a1a1"
        self.password = "salt"

    def create(self, name, password, email, tenant_id, enabled):
        """
        Mock of create method
        """
        user = MockUsers()
        user.name = name
        user.password = password
        user.email = email
        user.enabled = enabled
        self.tenant_id = tenant_id
        return user

    def get(self, user_id):
        """
        Mock of get method
        """
        user = MockUsers()
        if self.flag == 1:
            user.id = None
            return user
        user.id = user_id
        return user

    @staticmethod
    def list():
        """
        Mock of list method
        """
        return [MockUsers()]

    @staticmethod
    def delete(user_id):
        """
        Mock of delete method
        """
        return user_id

    @staticmethod
    def update(user, name, email, enabled):
        """
        Mock of update method
        """
        return (user, name, email, enabled)

    @staticmethod
    def update_password(user, password):
        """
        Mock of update_password method
        """
        return (user, password)


class Unauthorized(Exception):
    """
    The base exception class for all exceptions.
    """

    def __init__(self, message="Test"):
        super(Unauthorized, self).__init__(message)
        self.msg = message


class AuthorizationFailure(Exception):
    """
    Additional exception class to Unauthorized.
    """

    def __init__(self, message="Test"):
        super(AuthorizationFailure, self).__init__(message)
        self.msg = message


class MockExceptions(object):
    """
    Mock of exceptions class
    """

    def __init__(self):
        self.Unauthorized = Unauthorized
        self.AuthorizationFailure = AuthorizationFailure


class MockKeystoneClient(object):
    """
    Mock of keystoneclient module
    """

    def __init__(self):
        self.exceptions = MockExceptions()


class MockClient(object):
    """
    Mock of Client class
    """

    flag = None

    def __init__(self, profile=None, **conn_args):
        self.ec2 = MockEC2()
        self.endpoints = MockEndpoints()
        self.services = MockServices()
        self.roles = MockRoles()
        self.tenants = MockTenants()
        self.service_catalog = MockServiceCatalog()
        self.users = MockUsers()

    def Client(self, **kwargs):
        """
        Mock of Client method
        """
        if self.flag == 1:
            raise Unauthorized
        return True


class KeystoneTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.keystone
    """

    def setup_loader_modules(self):
        return {
            keystone: {
                "auth": MockClient,
                "client": MockClient(),
                "keystoneclient": MockKeystoneClient(),
            }
        }

    # 'ec2_credentials_create' function tests: 1

    def test_ec2_credentials_create(self):
        """
        Test if it create EC2-compatible credentials for user per tenant
        """
        self.assertDictEqual(
            keystone.ec2_credentials_create(), {"Error": "Could not resolve User ID"}
        )

        self.assertDictEqual(
            keystone.ec2_credentials_create(user_id="salt"),
            {"Error": "Could not resolve Tenant ID"},
        )

        self.assertDictEqual(
            keystone.ec2_credentials_create(user_id="salt", tenant_id="72278"),
            {"access": "", "tenant_id": "72278", "secret": "", "user_id": "salt"},
        )

    # 'ec2_credentials_delete' function tests: 1

    def test_ec2_credentials_delete(self):
        """
        Test if it delete EC2-compatible credentials
        """
        self.assertDictEqual(
            keystone.ec2_credentials_delete(), {"Error": "Could not resolve User ID"}
        )

        self.assertEqual(
            keystone.ec2_credentials_delete(user_id="salt", access_key="72278"),
            'ec2 key "72278" deleted under user id "salt"',
        )

    # 'ec2_credentials_get' function tests: 1

    def test_ec2_credentials_get(self):
        """
        Test if it return ec2_credentials for a user
        (keystone ec2-credentials-get)
        """
        self.assertDictEqual(
            keystone.ec2_credentials_get(), {"Error": "Unable to resolve user id"}
        )

        self.assertDictEqual(
            keystone.ec2_credentials_get(user_id="salt"),
            {"Error": "Access key is required"},
        )

        self.assertDictEqual(
            keystone.ec2_credentials_get(
                user_id="salt", access="72278", profile="openstack1"
            ),
            {
                "salt": {
                    "access": "72278",
                    "secret": "",
                    "tenant": "",
                    "user_id": "salt",
                }
            },
        )

    # 'ec2_credentials_list' function tests: 1

    def test_ec2_credentials_list(self):
        """
        Test if it return a list of ec2_credentials
        for a specific user (keystone ec2-credentials-list)
        """
        self.assertDictEqual(
            keystone.ec2_credentials_list(), {"Error": "Unable to resolve user id"}
        )

        self.assertDictEqual(
            keystone.ec2_credentials_list(user_id="salt", profile="openstack1"),
            {"salt": {"access": "", "secret": "", "tenant_id": "", "user_id": "salt"}},
        )

    # 'endpoint_get' function tests: 1

    def test_endpoint_get(self):
        """
        Test if it return a specific endpoint (keystone endpoint-get)
        """
        self.assertDictEqual(
            keystone.endpoint_get("nova", "RegionOne", profile="openstack"),
            {"Error": "Could not find the specified service"},
        )

        ret = {"Error": "Could not find endpoint for the specified service"}
        MockServices.flag = 1
        self.assertDictEqual(
            keystone.endpoint_get("iptables", "RegionOne", profile="openstack"), ret
        )

        MockServices.flag = 0
        self.assertDictEqual(
            keystone.endpoint_get("iptables", "RegionOne", profile="openstack"),
            {
                "adminurl": "adminurl",
                "id": "007",
                "internalurl": "internalurl",
                "publicurl": "publicurl",
                "region": "RegionOne",
                "service_id": "117",
            },
        )

    # 'endpoint_list' function tests: 1

    def test_endpoint_list(self):
        """
        Test if it return a list of available endpoints
        (keystone endpoints-list)
        """
        self.assertDictEqual(
            keystone.endpoint_list(profile="openstack1"),
            {
                "007": {
                    "adminurl": "adminurl",
                    "id": "007",
                    "internalurl": "internalurl",
                    "publicurl": "publicurl",
                    "region": "RegionOne",
                    "service_id": "117",
                }
            },
        )

    # 'endpoint_create' function tests: 1

    def test_endpoint_create(self):
        """
        Test if it create an endpoint for an Openstack service
        """
        self.assertDictEqual(
            keystone.endpoint_create("nova"),
            {"Error": "Could not find the specified service"},
        )

        MockServices.flag = 2
        self.assertDictEqual(
            keystone.endpoint_create(
                "iptables",
                "http://public/url",
                "http://internal/url",
                "http://adminurl/url",
                "RegionOne",
            ),
            {
                "adminurl": "adminurl",
                "id": "007",
                "internalurl": "internalurl",
                "publicurl": "publicurl",
                "region": "RegionOne",
                "service_id": "117",
            },
        )

    # 'endpoint_delete' function tests: 1

    def test_endpoint_delete(self):
        """
        Test if it delete an endpoint for an Openstack service
        """
        ret = {"Error": "Could not find any endpoints for the service"}
        self.assertDictEqual(keystone.endpoint_delete("nova", "RegionOne"), ret)

        with patch.object(
            keystone, "endpoint_get", MagicMock(side_effect=[{"id": "117"}, None])
        ):
            self.assertTrue(keystone.endpoint_delete("iptables", "RegionOne"))

    # 'role_create' function tests: 1

    def test_role_create(self):
        """
        Test if it create named role
        """
        self.assertDictEqual(
            keystone.role_create("nova"), {"Error": 'Role "nova" already exists'}
        )

        self.assertDictEqual(
            keystone.role_create("iptables"), {"Error": "Unable to resolve role id"}
        )

    # 'role_delete' function tests: 1

    def test_role_delete(self):
        """
        Test if it delete a role (keystone role-delete)
        """
        self.assertDictEqual(
            keystone.role_delete(), {"Error": "Unable to resolve role id"}
        )

        self.assertEqual(keystone.role_delete("iptables"), "Role ID iptables deleted")

    # 'role_get' function tests: 1

    def test_role_get(self):
        """
        Test if it return a specific roles (keystone role-get)
        """
        self.assertDictEqual(
            keystone.role_get(), {"Error": "Unable to resolve role id"}
        )

        self.assertDictEqual(
            keystone.role_get(name="nova"), {"nova": {"id": "113", "name": "nova"}}
        )

    # 'role_list' function tests: 1

    def test_role_list(self):
        """
        Test if it return a list of available roles (keystone role-list)
        """
        self.assertDictEqual(
            keystone.role_list(),
            {
                "nova": {
                    "id": "113",
                    "name": "nova",
                    "tenant_id": "a1a1",
                    "user_id": "446",
                }
            },
        )

    # 'service_create' function tests: 1

    def test_service_create(self):
        """
        Test if it add service to Keystone service catalog
        """
        MockServices.flag = 2
        self.assertDictEqual(
            keystone.service_create("nova", "compute", "OpenStack Service"),
            {
                "iptables": {
                    "description": "description",
                    "id": "005",
                    "name": "iptables",
                    "type": "type",
                }
            },
        )

    # 'service_delete' function tests: 1

    def test_service_delete(self):
        """
        Test if it delete a service from Keystone service catalog
        """
        self.assertEqual(
            keystone.service_delete("iptables"),
            'Keystone service ID "iptables" deleted',
        )

    # 'service_get' function tests: 1

    def test_service_get(self):
        """
        Test if it return a list of available services (keystone services-list)
        """
        MockServices.flag = 0
        self.assertDictEqual(
            keystone.service_get(), {"Error": "Unable to resolve service id"}
        )

        MockServices.flag = 2
        self.assertDictEqual(
            keystone.service_get(service_id="c965"),
            {
                "iptables": {
                    "description": "description",
                    "id": "c965",
                    "name": "iptables",
                    "type": "type",
                }
            },
        )

    # 'service_list' function tests: 1

    def test_service_list(self):
        """
        Test if it return a list of available services (keystone services-list)
        """
        MockServices.flag = 0
        self.assertDictEqual(
            keystone.service_list(profile="openstack1"),
            {
                "iptables": {
                    "description": "description",
                    "id": "117",
                    "name": "iptables",
                    "type": "type",
                }
            },
        )

    # 'tenant_create' function tests: 1

    def test_tenant_create(self):
        """
        Test if it create a keystone tenant
        """
        self.assertDictEqual(
            keystone.tenant_create("nova"),
            {
                "nova": {
                    "description": "description",
                    "id": "446",
                    "name": "nova",
                    "enabled": "True",
                }
            },
        )

    # 'tenant_delete' function tests: 1

    def test_tenant_delete(self):
        """
        Test if it delete a tenant (keystone tenant-delete)
        """
        self.assertDictEqual(
            keystone.tenant_delete(), {"Error": "Unable to resolve tenant id"}
        )

        self.assertEqual(keystone.tenant_delete("nova"), "Tenant ID nova deleted")

    # 'tenant_get' function tests: 1

    def test_tenant_get(self):
        """
        Test if it return a specific tenants (keystone tenant-get)
        """
        self.assertDictEqual(
            keystone.tenant_get(), {"Error": "Unable to resolve tenant id"}
        )

        self.assertDictEqual(
            keystone.tenant_get(tenant_id="446"),
            {
                "nova": {
                    "description": "description",
                    "id": "446",
                    "name": "nova",
                    "enabled": "True",
                }
            },
        )

    # 'tenant_list' function tests: 1

    def test_tenant_list(self):
        """
        Test if it return a list of available tenants (keystone tenants-list)
        """
        self.assertDictEqual(
            keystone.tenant_list(),
            {
                "nova": {
                    "description": "description",
                    "id": "446",
                    "name": "nova",
                    "enabled": "True",
                }
            },
        )

    # 'tenant_update' function tests: 1

    def test_tenant_update(self):
        """
        Test if it update a tenant's information (keystone tenant-update)
        """
        self.assertDictEqual(
            keystone.tenant_update(), {"Error": "Unable to resolve tenant id"}
        )

    # 'token_get' function tests: 1

    def test_token_get(self):
        """
        Test if it return the configured tokens (keystone token-get)
        """
        self.assertDictEqual(
            keystone.token_get(),
            {"expires": "No", "id": "446", "tenant_id": "ae04", "user_id": "admin"},
        )

    # 'user_list' function tests: 1

    def test_user_list(self):
        """
        Test if it return a list of available users (keystone user-list)
        """
        self.assertDictEqual(
            keystone.user_list(),
            {
                "nova": {
                    "name": "nova",
                    "tenant_id": "a1a1",
                    "enabled": "True",
                    "id": "446",
                    "password": "salt",
                    "email": "salt@saltstack.com",
                }
            },
        )

    # 'user_get' function tests: 1

    def test_user_get(self):
        """
        Test if it return a specific users (keystone user-get)
        """
        self.assertDictEqual(
            keystone.user_get(), {"Error": "Unable to resolve user id"}
        )

        self.assertDictEqual(
            keystone.user_get(user_id="446"),
            {
                "nova": {
                    "name": "nova",
                    "tenant_id": "a1a1",
                    "enabled": "True",
                    "id": "446",
                    "password": "salt",
                    "email": "salt@saltstack.com",
                }
            },
        )

    # 'user_create' function tests: 1

    def test_user_create(self):
        """
        Test if it create a user (keystone user-create)
        """
        self.assertDictEqual(
            keystone.user_create(
                name="nova",
                password="salt",
                email="salt@saltstack.com",
                tenant_id="a1a1",
            ),
            {
                "nova": {
                    "name": "nova",
                    "tenant_id": "a1a1",
                    "enabled": "True",
                    "id": "446",
                    "password": "salt",
                    "email": "salt@saltstack.com",
                }
            },
        )

    # 'user_delete' function tests: 1

    def test_user_delete(self):
        """
        Test if it delete a user (keystone user-delete)
        """
        self.assertDictEqual(
            keystone.user_delete(), {"Error": "Unable to resolve user id"}
        )

        self.assertEqual(keystone.user_delete("nova"), "User ID nova deleted")

    # 'user_update' function tests: 1

    def test_user_update(self):
        """
        Test if it update a user's information (keystone user-update)
        """
        self.assertDictEqual(
            keystone.user_update(), {"Error": "Unable to resolve user id"}
        )

        self.assertEqual(keystone.user_update("nova"), "Info updated for user ID nova")

    # 'user_verify_password' function tests: 1

    def test_user_verify_password(self):
        """
        Test if it verify a user's password
        """
        mock = MagicMock(return_value="http://127.0.0.1:35357/v2.0")
        with patch.dict(keystone.__salt__, {"config.option": mock}):
            self.assertDictEqual(
                keystone.user_verify_password(),
                {"Error": "Unable to resolve user name"},
            )

            self.assertTrue(keystone.user_verify_password(user_id="446", name="nova"))

            MockClient.flag = 1
            self.assertFalse(keystone.user_verify_password(user_id="446", name="nova"))

    # 'user_password_update' function tests: 1

    def test_user_password_update(self):
        """
        Test if it update a user's password (keystone user-password-update)
        """
        self.assertDictEqual(
            keystone.user_password_update(), {"Error": "Unable to resolve user id"}
        )

        self.assertEqual(
            keystone.user_password_update("nova"), "Password updated for user ID nova"
        )

    # 'user_role_add' function tests: 1

    def test_user_role_add(self):
        """
        Test if it add role for user in tenant (keystone user-role-add)
        """
        self.assertEqual(
            keystone.user_role_add(user="nova", tenant="nova", role="nova"),
            '"nova" role added for user "nova" for "nova" tenant/project',
        )

        MockRoles.flag = 1
        self.assertDictEqual(
            keystone.user_role_add(user="nova", tenant="nova", role="nova"),
            {"Error": "Unable to resolve role id"},
        )

        MockTenants.flag = 1
        self.assertDictEqual(
            keystone.user_role_add(user="nova", tenant="nova"),
            {"Error": "Unable to resolve tenant/project id"},
        )

        MockUsers.flag = 1
        self.assertDictEqual(
            keystone.user_role_add(user="nova"), {"Error": "Unable to resolve user id"}
        )

    # 'user_role_remove' function tests: 1

    def test_user_role_remove(self):
        """
        Test if it add role for user in tenant (keystone user-role-add)
        """
        MockUsers.flag = 1
        self.assertDictEqual(
            keystone.user_role_remove(user="nova"),
            {"Error": "Unable to resolve user id"},
        )

        MockUsers.flag = 0
        MockTenants.flag = 1
        self.assertDictEqual(
            keystone.user_role_remove(user="nova", tenant="nova"),
            {"Error": "Unable to resolve tenant/project id"},
        )

        MockTenants.flag = 0
        MockRoles.flag = 1
        self.assertDictEqual(
            keystone.user_role_remove(user="nova", tenant="nova", role="nova"),
            {"Error": "Unable to resolve role id"},
        )

        ret = '"nova" role removed for user "nova" under "nova" tenant'
        MockRoles.flag = 0
        self.assertEqual(
            keystone.user_role_remove(user="nova", tenant="nova", role="nova"), ret
        )

    # 'user_role_list' function tests: 1

    def test_user_role_list(self):
        """
        Test if it return a list of available user_roles
        (keystone user-roles-list)
        """
        self.assertDictEqual(
            keystone.user_role_list(user="nova"),
            {"Error": "Unable to resolve user or tenant/project id"},
        )

        self.assertDictEqual(
            keystone.user_role_list(user_name="nova", tenant_name="nova"),
            {
                "nova": {
                    "id": "113",
                    "name": "nova",
                    "tenant_id": "446",
                    "user_id": "446",
                }
            },
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.states.grafana as grafana

# Import Salt Libs
import salt.utils.json
from salt.exceptions import SaltInvocationError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class GrafanaTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.grafana
    """

    def setup_loader_modules(self):
        return {grafana: {}}

    # 'dashboard_present' function tests: 1

    def test_dashboard_present(self):
        """
        Test to ensure the grafana dashboard exists and is managed.
        """
        name = "myservice"
        rows = ["systemhealth", "requests", "title"]
        row = [{"panels": [{"id": "a"}], "title": "systemhealth"}]

        ret = {"name": name, "result": None, "changes": {}, "comment": ""}

        comt1 = (
            "Dashboard myservice is set to be updated. The following rows "
            "set to be updated: {0}".format(["systemhealth"])
        )
        self.assertRaises(
            SaltInvocationError, grafana.dashboard_present, name, profile=False
        )

        self.assertRaises(
            SaltInvocationError, grafana.dashboard_present, name, True, True
        )

        mock = MagicMock(
            side_effect=[
                {"hosts": True, "index": False},
                {"hosts": True, "index": True},
                {"hosts": True, "index": True},
                {"hosts": True, "index": True},
                {"hosts": True, "index": True},
                {"hosts": True, "index": True},
                {"hosts": True, "index": True},
            ]
        )
        mock_f = MagicMock(side_effect=[False, False, True, True, True, True])
        mock_t = MagicMock(return_value="")
        mock_i = MagicMock(return_value=False)
        source = {"dashboard": '["rows", {"rows":["baz", null, 1.0, 2]}]'}
        mock_dict = MagicMock(return_value={"_source": source})
        with patch.dict(
            grafana.__salt__,
            {
                "config.option": mock,
                "elasticsearch.exists": mock_f,
                "pillar.get": mock_t,
                "elasticsearch.get": mock_dict,
                "elasticsearch.index": mock_i,
            },
        ):
            self.assertRaises(SaltInvocationError, grafana.dashboard_present, name)

            with patch.dict(grafana.__opts__, {"test": True}):
                self.assertRaises(SaltInvocationError, grafana.dashboard_present, name)

                comt = "Dashboard {0} is set to be created.".format(name)
                ret.update({"comment": comt})
                self.assertDictEqual(grafana.dashboard_present(name, True), ret)

                mock = MagicMock(
                    return_value={"rows": [{"panels": "b", "title": "systemhealth"}]}
                )
                with patch.object(salt.utils.json, "loads", mock):
                    ret.update({"comment": comt1, "result": None})
                    self.assertDictEqual(
                        grafana.dashboard_present(name, True, rows=row), ret
                    )

            with patch.object(
                salt.utils.json, "loads", MagicMock(return_value={"rows": {}})
            ):
                self.assertRaises(
                    SaltInvocationError,
                    grafana.dashboard_present,
                    name,
                    rows_from_pillar=rows,
                )

                comt = "Dashboard myservice is up to date"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(grafana.dashboard_present(name, True), ret)

            mock = MagicMock(
                return_value={"rows": [{"panels": "b", "title": "systemhealth"}]}
            )
            with patch.dict(grafana.__opts__, {"test": False}):
                with patch.object(salt.utils.json, "loads", mock):
                    comt = "Failed to update dashboard myservice."
                    ret.update({"comment": comt, "result": False})
                    self.assertDictEqual(
                        grafana.dashboard_present(name, True, rows=row), ret
                    )

    # 'dashboard_absent' function tests: 1

    def test_dashboard_absent(self):
        """
        Test to ensure the named grafana dashboard is deleted.
        """
        name = "myservice"

        ret = {"name": name, "result": None, "changes": {}, "comment": ""}

        mock = MagicMock(
            side_effect=[
                {"hosts": True, "index": False},
                {"hosts": True, "index": True},
                {"hosts": True, "index": True},
            ]
        )
        mock_f = MagicMock(side_effect=[True, False])
        with patch.dict(
            grafana.__salt__, {"config.option": mock, "elasticsearch.exists": mock_f}
        ):
            self.assertRaises(SaltInvocationError, grafana.dashboard_absent, name)

            with patch.dict(grafana.__opts__, {"test": True}):
                comt = "Dashboard myservice is set to be removed."
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(grafana.dashboard_absent(name), ret)

            comt = "Dashboard myservice does not exist."
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(grafana.dashboard_absent(name), ret)



# -*- coding: utf-8 -*-
"""
    :synopsis: Unit Tests for Windows SNMP Module 'state.win_snmp'
    :platform: Windows
    :maturity: develop
    .. versionadded:: 2017.7.0
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.win_snmp as win_snmp
from salt.ext import six

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class WinSnmpTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.win_snmp
    """

    def setup_loader_modules(self):
        return {win_snmp: {}}

    def test_agent_settings(self):
        """
        Test - Manage the SNMP sysContact, sysLocation, and sysServices settings.
        """
        kwargs = {
            "name": "agent-settings",
            "contact": "TestContact",
            "location": "TestLocation",
            "services": ["Internet"],
        }
        ret = {
            "name": kwargs["name"],
            "changes": {},
            "comment": "Agent settings already contain the provided values.",
            "result": True,
        }
        # Using this instead of dictionary comprehension in order to make pylint happy.
        get_ret = dict(
            (key, value) for (key, value) in six.iteritems(kwargs) if key != "name"
        )
        mock_value_get = MagicMock(return_value=get_ret)
        mock_value_set = MagicMock(return_value=True)
        with patch.dict(
            win_snmp.__salt__,
            {
                "win_snmp.get_agent_settings": mock_value_get,
                "win_snmp.set_agent_settings": mock_value_set,
            },
        ):
            with patch.dict(win_snmp.__opts__, {"test": False}):
                self.assertEqual(win_snmp.agent_settings(**kwargs), ret)

    def test_auth_traps_enabled(self):
        """
        Test - Manage the sending of authentication traps.
        """
        kwargs = {"name": "auth-traps", "status": True}
        ret = {
            "name": kwargs["name"],
            "changes": {"old": False, "new": True},
            "comment": "Set EnableAuthenticationTraps to contain the provided value.",
            "result": True,
        }
        mock_value_get = MagicMock(return_value=False)
        mock_value_set = MagicMock(return_value=True)
        with patch.dict(
            win_snmp.__salt__,
            {
                "win_snmp.get_auth_traps_enabled": mock_value_get,
                "win_snmp.set_auth_traps_enabled": mock_value_set,
            },
        ):
            with patch.dict(win_snmp.__opts__, {"test": False}):
                self.assertEqual(win_snmp.auth_traps_enabled(**kwargs), ret)
            with patch.dict(win_snmp.__opts__, {"test": True}):
                ret["comment"] = "EnableAuthenticationTraps will be changed."
                ret["result"] = None
                self.assertEqual(win_snmp.auth_traps_enabled(**kwargs), ret)

    def test_community_names(self):
        """
        Test - Manage the SNMP accepted community names and their permissions.
        """
        kwargs = {
            "name": "community-names",
            "communities": {"TestCommunity": "Read Create"},
        }
        ret = {
            "name": kwargs["name"],
            "changes": {},
            "comment": "Communities already contain the provided values.",
            "result": True,
        }
        mock_value_get = MagicMock(return_value=kwargs["communities"])
        mock_value_set = MagicMock(return_value=True)
        with patch.dict(
            win_snmp.__salt__,
            {
                "win_snmp.get_community_names": mock_value_get,
                "win_snmp.set_community_names": mock_value_set,
            },
        ):
            with patch.dict(win_snmp.__opts__, {"test": False}):
                self.assertEqual(win_snmp.community_names(**kwargs), ret)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import random
import string

# Import Salt libs
import salt.config
import salt.loader
import salt.states.boto_lambda as boto_lambda
import salt.utils.json

# Import 3rd-party libs
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
from salt.utils.versions import LooseVersion
from tests.support.helpers import slowTest

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# pylint: disable=import-error,no-name-in-module
from tests.unit.modules.test_boto_lambda import BotoLambdaTestCaseMixin, TempZipFile

# Import test suite libs


try:
    import boto3
    from botocore.exceptions import ClientError
    from botocore import __version__ as found_botocore_version

    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False

# pylint: enable=import-error,no-name-in-module

# the boto_lambda module relies on the connect_to_region() method
# which was added in boto 2.8.0
# https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
required_boto3_version = "1.2.1"
required_botocore_version = "1.5.2"

region = "us-east-1"
access_key = "GKTADJGHEIQSXMKKRBJ08H"
secret_key = "askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs"
conn_parameters = {
    "region": region,
    "key": access_key,
    "keyid": secret_key,
    "profile": {},
}
error_message = (
    "An error occurred (101) when calling the {0} operation: Test-defined error"
)
error_content = {"Error": {"Code": 101, "Message": "Test-defined error"}}
function_ret = dict(
    FunctionName="testfunction",
    Runtime="python2.7",
    Role="arn:aws:iam::1234:role/functionrole",
    Handler="handler",
    Description="abcdefg",
    Timeout=5,
    MemorySize=128,
    CodeSha256="abcdef",
    CodeSize=199,
    FunctionArn="arn:lambda:us-east-1:1234:Something",
    LastModified="yes",
    VpcConfig={"SubnetIds": [], "SecurityGroupIds": []},
)
alias_ret = dict(
    AliasArn="arn:lambda:us-east-1:1234:Something",
    Name="testalias",
    FunctionVersion="3",
    Description="Alias description",
)
event_source_mapping_ret = dict(
    UUID="1234-1-123",
    BatchSize=123,
    EventSourceArn="arn:aws:dynamodb:us-east-1:1234::Something",
    FunctionArn="arn:aws:lambda:us-east-1:1234:function:myfunc",
    LastModified="yes",
    LastProcessingResult="SUCCESS",
    State="Enabled",
    StateTransitionReason="Random",
)

log = logging.getLogger(__name__)


def _has_required_boto():
    """
    Returns True/False boolean depending on if Boto is installed and correct
    version.
    """
    if not HAS_BOTO:
        return False
    elif LooseVersion(boto3.__version__) < LooseVersion(required_boto3_version):
        return False
    elif LooseVersion(found_botocore_version) < LooseVersion(required_botocore_version):
        return False
    else:
        return True


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    (
        "The boto3 module must be greater than or equal to version {0}, "
        "and botocore must be greater than or equal to {1}".format(
            required_boto3_version, required_botocore_version
        )
    ),
)
class BotoLambdaStateTestCaseBase(TestCase, LoaderModuleMockMixin):
    conn = None

    def setup_loader_modules(self):
        ctx = {}
        utils = salt.loader.utils(
            self.opts,
            whitelist=["boto", "boto3", "args", "systemd", "path", "platform", "reg"],
            context=ctx,
        )
        serializers = salt.loader.serializers(self.opts)
        self.funcs = funcs = salt.loader.minion_mods(
            self.opts, context=ctx, utils=utils, whitelist=["boto_lambda"]
        )
        self.salt_states = salt.loader.states(
            opts=self.opts,
            functions=funcs,
            utils=utils,
            whitelist=["boto_lambda"],
            serializers=serializers,
        )
        return {
            boto_lambda: {
                "__opts__": self.opts,
                "__salt__": funcs,
                "__utils__": utils,
                "__states__": self.salt_states,
                "__serializers__": serializers,
            }
        }

    @classmethod
    def setUpClass(cls):
        cls.opts = salt.config.DEFAULT_MINION_OPTS.copy()
        cls.opts["grains"] = salt.loader.grains(cls.opts)

    @classmethod
    def tearDownClass(cls):
        del cls.opts

    def setUp(self):
        self.addCleanup(delattr, self, "funcs")
        self.addCleanup(delattr, self, "salt_states")
        # connections keep getting cached from prior tests, can't find the
        # correct context object to clear it. So randomize the cache key, to prevent any
        # cache hits
        conn_parameters["key"] = "".join(
            random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
        )

        self.patcher = patch("boto3.session.Session")
        self.addCleanup(self.patcher.stop)
        self.addCleanup(delattr, self, "patcher")
        mock_session = self.patcher.start()

        session_instance = mock_session.return_value
        self.conn = MagicMock()
        self.addCleanup(delattr, self, "conn")
        session_instance.client.return_value = self.conn


class BotoLambdaFunctionTestCase(BotoLambdaStateTestCaseBase, BotoLambdaTestCaseMixin):
    """
    TestCase for salt.modules.boto_lambda state.module
    """

    @slowTest
    def test_present_when_function_does_not_exist(self):
        """
        Tests present on a function that does not exist.
        """
        self.conn.list_functions.side_effect = [
            {"Functions": []},
            {"Functions": [function_ret]},
        ]
        self.conn.create_function.return_value = function_ret
        with patch.dict(
            self.funcs, {"boto_iam.get_account_id": MagicMock(return_value="1234")}
        ):
            with TempZipFile() as zipfile:
                result = self.salt_states["boto_lambda.function_present"](
                    "function present",
                    FunctionName=function_ret["FunctionName"],
                    Runtime=function_ret["Runtime"],
                    Role=function_ret["Role"],
                    Handler=function_ret["Handler"],
                    ZipFile=zipfile,
                )

        self.assertTrue(result["result"])
        self.assertEqual(
            result["changes"]["new"]["function"]["FunctionName"],
            function_ret["FunctionName"],
        )

    @slowTest
    def test_present_when_function_exists(self):
        self.conn.list_functions.return_value = {"Functions": [function_ret]}
        self.conn.update_function_code.return_value = function_ret

        with patch.dict(
            self.funcs, {"boto_iam.get_account_id": MagicMock(return_value="1234")}
        ):
            with TempZipFile() as zipfile:
                with patch("hashlib.sha256") as sha256:
                    with patch("os.path.getsize", return_value=199):
                        sha = sha256()
                        digest = sha.digest()
                        encoded = sha.encode()
                        encoded.strip.return_value = function_ret["CodeSha256"]
                        result = self.salt_states["boto_lambda.function_present"](
                            "function present",
                            FunctionName=function_ret["FunctionName"],
                            Runtime=function_ret["Runtime"],
                            Role=function_ret["Role"],
                            Handler=function_ret["Handler"],
                            ZipFile=zipfile,
                            Description=function_ret["Description"],
                            Timeout=function_ret["Timeout"],
                        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    @slowTest
    def test_present_with_failure(self):
        self.conn.list_functions.side_effect = [
            {"Functions": []},
            {"Functions": [function_ret]},
        ]
        self.conn.create_function.side_effect = ClientError(
            error_content, "create_function"
        )
        with patch.dict(
            self.funcs, {"boto_iam.get_account_id": MagicMock(return_value="1234")}
        ):
            with TempZipFile() as zipfile:
                with patch("hashlib.sha256") as sha256:
                    with patch("os.path.getsize", return_value=199):
                        sha = sha256()
                        digest = sha.digest()
                        encoded = sha.encode()
                        encoded.strip.return_value = function_ret["CodeSha256"]
                        result = self.salt_states["boto_lambda.function_present"](
                            "function present",
                            FunctionName=function_ret["FunctionName"],
                            Runtime=function_ret["Runtime"],
                            Role=function_ret["Role"],
                            Handler=function_ret["Handler"],
                            ZipFile=zipfile,
                            Description=function_ret["Description"],
                            Timeout=function_ret["Timeout"],
                        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])

    def test_absent_when_function_does_not_exist(self):
        """
        Tests absent on a function that does not exist.
        """
        self.conn.list_functions.return_value = {"Functions": [function_ret]}
        result = self.salt_states["boto_lambda.function_absent"]("test", "myfunc")
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_absent_when_function_exists(self):
        self.conn.list_functions.return_value = {"Functions": [function_ret]}
        result = self.salt_states["boto_lambda.function_absent"](
            "test", function_ret["FunctionName"]
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"]["new"]["function"], None)

    def test_absent_with_failure(self):
        self.conn.list_functions.return_value = {"Functions": [function_ret]}
        self.conn.delete_function.side_effect = ClientError(
            error_content, "delete_function"
        )
        result = self.salt_states["boto_lambda.function_absent"](
            "test", function_ret["FunctionName"]
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])

    @slowTest
    def test_present_when_function_exists_and_permissions(self):
        self.conn.list_functions.return_value = {"Functions": [function_ret]}
        self.conn.update_function_code.return_value = function_ret
        self.conn.get_policy.return_value = {
            "Policy": salt.utils.json.dumps(
                {
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Condition": {
                                "ArnLike": {
                                    "AWS:SourceArn": "arn:aws:events:us-east-1:9999999999:rule/fooo"
                                }
                            },
                            "Action": "lambda:InvokeFunction",
                            "Resource": "arn:aws:lambda:us-east-1:999999999999:function:testfunction",
                            "Effect": "Allow",
                            "Principal": {"Service": "events.amazonaws.com"},
                            "Sid": "AWSEvents_foo-bar999999999999",
                        }
                    ],
                    "Id": "default",
                }
            )
        }

        with patch.dict(
            self.funcs, {"boto_iam.get_account_id": MagicMock(return_value="1234")}
        ):
            with TempZipFile() as zipfile:
                with patch("hashlib.sha256") as sha256:
                    with patch("os.path.getsize", return_value=199):
                        sha = sha256()
                        digest = sha.digest()
                        encoded = sha.encode()
                        encoded.strip.return_value = function_ret["CodeSha256"]
                        result = self.salt_states["boto_lambda.function_present"](
                            "function present",
                            FunctionName=function_ret["FunctionName"],
                            Runtime=function_ret["Runtime"],
                            Role=function_ret["Role"],
                            Handler=function_ret["Handler"],
                            ZipFile=zipfile,
                            Description=function_ret["Description"],
                            Timeout=function_ret["Timeout"],
                        )
        self.assertTrue(result["result"])
        self.assertEqual(
            result["changes"],
            {
                "old": {
                    "Permissions": {
                        "AWSEvents_foo-bar999999999999": {
                            "Action": "lambda:InvokeFunction",
                            "Principal": "events.amazonaws.com",
                            "SourceArn": "arn:aws:events:us-east-1:9999999999:rule/fooo",
                        }
                    }
                },
                "new": {"Permissions": {"AWSEvents_foo-bar999999999999": {}}},
            },
        )


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    (
        "The boto3 module must be greater than or equal to version {0}, "
        "and botocore must be greater than or equal to {1}".format(
            required_boto3_version, required_botocore_version
        )
    ),
)
class BotoLambdaAliasTestCase(BotoLambdaStateTestCaseBase, BotoLambdaTestCaseMixin):
    """
    TestCase for salt.modules.boto_lambda state.module aliases
    """

    def test_present_when_alias_does_not_exist(self):
        """
        Tests present on a alias that does not exist.
        """
        self.conn.list_aliases.side_effect = [{"Aliases": []}, {"Aliases": [alias_ret]}]
        self.conn.create_alias.return_value = alias_ret
        result = self.salt_states["boto_lambda.alias_present"](
            "alias present",
            FunctionName="testfunc",
            Name=alias_ret["Name"],
            FunctionVersion=alias_ret["FunctionVersion"],
        )

        self.assertTrue(result["result"])
        self.assertEqual(result["changes"]["new"]["alias"]["Name"], alias_ret["Name"])

    def test_present_when_alias_exists(self):
        self.conn.list_aliases.return_value = {"Aliases": [alias_ret]}
        self.conn.create_alias.return_value = alias_ret
        result = self.salt_states["boto_lambda.alias_present"](
            "alias present",
            FunctionName="testfunc",
            Name=alias_ret["Name"],
            FunctionVersion=alias_ret["FunctionVersion"],
            Description=alias_ret["Description"],
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    @slowTest
    def test_present_with_failure(self):
        self.conn.list_aliases.side_effect = [{"Aliases": []}, {"Aliases": [alias_ret]}]
        self.conn.create_alias.side_effect = ClientError(error_content, "create_alias")
        result = self.salt_states["boto_lambda.alias_present"](
            "alias present",
            FunctionName="testfunc",
            Name=alias_ret["Name"],
            FunctionVersion=alias_ret["FunctionVersion"],
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])

    def test_absent_when_alias_does_not_exist(self):
        """
        Tests absent on a alias that does not exist.
        """
        self.conn.list_aliases.return_value = {"Aliases": [alias_ret]}
        result = self.salt_states["boto_lambda.alias_absent"](
            "alias absent", FunctionName="testfunc", Name="myalias"
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_absent_when_alias_exists(self):
        self.conn.list_aliases.return_value = {"Aliases": [alias_ret]}
        result = self.salt_states["boto_lambda.alias_absent"](
            "alias absent", FunctionName="testfunc", Name=alias_ret["Name"]
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"]["new"]["alias"], None)

    def test_absent_with_failure(self):
        self.conn.list_aliases.return_value = {"Aliases": [alias_ret]}
        self.conn.delete_alias.side_effect = ClientError(error_content, "delete_alias")
        result = self.salt_states["boto_lambda.alias_absent"](
            "alias absent", FunctionName="testfunc", Name=alias_ret["Name"]
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    (
        "The boto3 module must be greater than or equal to version {0}, "
        "and botocore must be greater than or equal to {1}".format(
            required_boto3_version, required_botocore_version
        )
    ),
)
class BotoLambdaEventSourceMappingTestCase(
    BotoLambdaStateTestCaseBase, BotoLambdaTestCaseMixin
):
    """
    TestCase for salt.modules.boto_lambda state.module event_source_mappings
    """

    def test_present_when_event_source_mapping_does_not_exist(self):
        """
        Tests present on a event_source_mapping that does not exist.
        """
        self.conn.list_event_source_mappings.side_effect = [
            {"EventSourceMappings": []},
            {"EventSourceMappings": [event_source_mapping_ret]},
        ]
        self.conn.get_event_source_mapping.return_value = event_source_mapping_ret
        self.conn.create_event_source_mapping.return_value = event_source_mapping_ret
        result = self.salt_states["boto_lambda.event_source_mapping_present"](
            "event source mapping present",
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName="myfunc",
            StartingPosition="LATEST",
        )

        self.assertTrue(result["result"])
        self.assertEqual(
            result["changes"]["new"]["event_source_mapping"]["UUID"],
            event_source_mapping_ret["UUID"],
        )

    def test_present_when_event_source_mapping_exists(self):
        self.conn.list_event_source_mappings.return_value = {
            "EventSourceMappings": [event_source_mapping_ret]
        }
        self.conn.get_event_source_mapping.return_value = event_source_mapping_ret
        self.conn.create_event_source_mapping.return_value = event_source_mapping_ret
        result = self.salt_states["boto_lambda.event_source_mapping_present"](
            "event source mapping present",
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            StartingPosition="LATEST",
            BatchSize=event_source_mapping_ret["BatchSize"],
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    @slowTest
    def test_present_with_failure(self):
        self.conn.list_event_source_mappings.side_effect = [
            {"EventSourceMappings": []},
            {"EventSourceMappings": [event_source_mapping_ret]},
        ]
        self.conn.get_event_source_mapping.return_value = event_source_mapping_ret
        self.conn.create_event_source_mapping.side_effect = ClientError(
            error_content, "create_event_source_mapping"
        )
        result = self.salt_states["boto_lambda.event_source_mapping_present"](
            "event source mapping present",
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            StartingPosition="LATEST",
            BatchSize=event_source_mapping_ret["BatchSize"],
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])

    def test_absent_when_event_source_mapping_does_not_exist(self):
        """
        Tests absent on a event_source_mapping that does not exist.
        """
        self.conn.list_event_source_mappings.return_value = {"EventSourceMappings": []}
        result = self.salt_states["boto_lambda.event_source_mapping_absent"](
            "event source mapping absent",
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName="myfunc",
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_absent_when_event_source_mapping_exists(self):
        self.conn.list_event_source_mappings.return_value = {
            "EventSourceMappings": [event_source_mapping_ret]
        }
        self.conn.get_event_source_mapping.return_value = event_source_mapping_ret
        result = self.salt_states["boto_lambda.event_source_mapping_absent"](
            "event source mapping absent",
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName="myfunc",
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"]["new"]["event_source_mapping"], None)

    def test_absent_with_failure(self):
        self.conn.list_event_source_mappings.return_value = {
            "EventSourceMappings": [event_source_mapping_ret]
        }
        self.conn.get_event_source_mapping.return_value = event_source_mapping_ret
        self.conn.delete_event_source_mapping.side_effect = ClientError(
            error_content, "delete_event_source_mapping"
        )
        result = self.salt_states["boto_lambda.event_source_mapping_absent"](
            "event source mapping absent",
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName="myfunc",
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import salt.config
import salt.version
from tests.support.case import MultimasterModuleCase
from tests.support.helpers import slowTest
from tests.support.mixins import AdaptedConfigurationTestCaseMixin


class TestModuleTest(MultimasterModuleCase, AdaptedConfigurationTestCaseMixin):
    """
    Validate the test module
    """

    def test_ping(self):
        """
        test.ping
        """
        self.assertEqual(self.run_function_all_masters("test.ping"), [True] * 2)

    def test_echo(self):
        """
        test.echo
        """
        self.assertEqual(
            self.run_function_all_masters("test.echo", ["text"]), ["text"] * 2
        )

    def test_version(self):
        """
        test.version
        """
        self.assertEqual(
            self.run_function_all_masters("test.version"),
            [salt.version.__saltstack_version__.string] * 2,
        )

    def test_conf_test(self):
        """
        test.conf_test
        """
        self.assertEqual(self.run_function_all_masters("test.conf_test"), ["baz"] * 2)

    def test_get_opts(self):
        """
        test.get_opts
        """
        opts = salt.config.minion_config(self.get_config_file_path("mm_minion"))
        ret = self.run_function_all_masters("test.get_opts")
        self.assertEqual(ret[0]["cachedir"], opts["cachedir"])
        self.assertEqual(ret[1]["cachedir"], opts["cachedir"])

    def test_cross_test(self):
        """
        test.cross_test
        """
        self.assertTrue(self.run_function_all_masters("test.cross_test", ["test.ping"]))

    def test_fib(self):
        """
        test.fib
        """
        ret = self.run_function_all_masters("test.fib", ["20"])
        self.assertEqual(ret[0][0], 6765)
        self.assertEqual(ret[1][0], 6765)

    @slowTest
    def test_collatz(self):
        """
        test.collatz
        """
        ret = self.run_function_all_masters("test.collatz", ["40"])
        self.assertEqual(ret[0][0][-1], 2)
        self.assertEqual(ret[1][0][-1], 2)

    def test_outputter(self):
        """
        test.outputter
        """
        self.assertEqual(
            self.run_function_all_masters("test.outputter", ["text"]), ["text"] * 2
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Thayne Harbaugh (tharbaug@adobe.com)

    tests.integration.shell.proxy
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

from __future__ import absolute_import, print_function, unicode_literals

import logging

import pytest
import salt.utils.platform
import tests.integration.utils
from tests.integration.utils import testprogram
from tests.support.helpers import slowTest
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


@pytest.mark.windows_whitelisted
class ProxyTest(testprogram.TestProgramCase):
    """
    Various integration tests for the salt-proxy executable.
    """

    @slowTest
    def test_exit_status_no_proxyid(self):
        """
        Ensure correct exit status when --proxyid argument is missing.
        """

        proxy = testprogram.TestDaemonSaltProxy(
            name="proxy-no_proxyid", parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        proxy.setup()
        # Needed due to verbatim_args=True
        args = ["--config-dir", proxy.abs_path(proxy.config_dir)]
        if not salt.utils.platform.is_windows():
            args.append("-d")
        stdout, stderr, status = proxy.run(
            args=args,
            # verbatim_args prevents --proxyid from being added automatically
            verbatim_args=True,
            catch_stderr=True,
            with_retcode=True,
            # The proxy minion had a bug where it would loop forever
            # without daemonizing - protect that with a timeout.
            timeout=60,
        )
        try:
            self.assert_exit_status(
                status,
                "EX_USAGE",
                message="no --proxyid specified",
                stdout=stdout,
                stderr=tests.integration.utils.decode_byte_list(stderr),
            )
        finally:
            # Although the start-up should fail, call shutdown() to set the
            # internal _shutdown flag and avoid the registered atexit calls to
            # cause timeout exceptions and respective traceback
            proxy.shutdown()

    # Hangs on Windows. You can add a timeout to the proxy.run command, but then
    # it just times out.
    @skipIf(salt.utils.platform.is_windows(), "Test hangs on Windows")
    @slowTest
    def test_exit_status_unknown_user(self):
        """
        Ensure correct exit status when the proxy is configured to run as an
        unknown user.
        """

        proxy = testprogram.TestDaemonSaltProxy(
            name="proxy-unknown_user",
            config_base={"user": "some_unknown_user_xyz"},
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        proxy.setup()
        stdout, stderr, status = proxy.run(
            args=["-d"] if not salt.utils.platform.is_windows() else [],
            catch_stderr=True,
            with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status,
                "EX_NOUSER",
                message="unknown user not on system",
                stdout=stdout,
                stderr=tests.integration.utils.decode_byte_list(stderr),
            )
        finally:
            # Although the start-up should fail, call shutdown() to set the
            # internal _shutdown flag and avoid the registered atexit calls to
            # cause timeout exceptions and respective traceback
            proxy.shutdown()

    @slowTest
    def test_exit_status_unknown_argument(self):
        """
        Ensure correct exit status when an unknown argument is passed to
        salt-proxy.
        """

        proxy = testprogram.TestDaemonSaltProxy(
            name="proxy-unknown_argument", parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        proxy.setup()
        args = ["--unknown-argument"]
        if not salt.utils.platform.is_windows():
            args.append("-b")
        stdout, stderr, status = proxy.run(
            args=args, catch_stderr=True, with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status,
                "EX_USAGE",
                message="unknown argument",
                stdout=stdout,
                stderr=stderr,
            )
        finally:
            # Although the start-up should fail, call shutdown() to set the
            # internal _shutdown flag and avoid the registered atexit calls to
            # cause timeout exceptions and respective traceback
            proxy.shutdown()

    # Hangs on Windows. You can add a timeout to the proxy.run command, but then
    # it just times out.
    @skipIf(salt.utils.platform.is_windows(), "Test hangs on Windows")
    @slowTest
    def test_exit_status_correct_usage(self):
        """
        Ensure correct exit status when salt-proxy starts correctly.

        Skip on Windows because daemonization not supported
        """
        proxy = testprogram.TestDaemonSaltProxy(
            name="proxy-correct_usage", parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        proxy.setup()
        stdout, stderr, status = proxy.run(
            args=["-d"] if not salt.utils.platform.is_windows() else [],
            catch_stderr=True,
            with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status,
                "EX_OK",
                message="correct usage",
                stdout=stdout,
                stderr=tests.integration.utils.decode_byte_list(stderr),
            )
        finally:
            proxy.shutdown(wait_for_orphans=3)



# -*- coding: utf-8 -*-
"""
Unit tests for salt._compat
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import sys

# Import Salt libs
import salt._compat as compat

# Import 3rd Party libs
from salt.ext.six import binary_type, text_type

# Import Salt Testing libs
from tests.support.unit import TestCase

log = logging.getLogger(__name__)
PY3 = sys.version_info.major == 3


class CompatTestCase(TestCase):
    def test_text(self):
        ret = compat.text_("test string")
        self.assertTrue(isinstance(ret, text_type))

    def test_text_binary(self):
        ret = compat.text_(b"test string")
        self.assertTrue(isinstance(ret, text_type))

    def test_bytes(self):
        ret = compat.bytes_("test string")
        self.assertTrue(isinstance(ret, binary_type))

    def test_bytes_binary(self):
        ret = compat.bytes_(b"test string")
        self.assertTrue(isinstance(ret, binary_type))

    def test_ascii_native(self):
        ret = compat.ascii_native_("test string")
        self.assertTrue(isinstance(ret, str))

    def test_ascii_native_binary(self):
        ret = compat.ascii_native_(b"test string")
        self.assertTrue(isinstance(ret, str))

    def test_native(self):
        ret = compat.native_("test string")
        self.assertTrue(isinstance(ret, str))

    def test_native_binary(self):
        ret = compat.native_(b"test string")
        self.assertTrue(isinstance(ret, str))

    def test_string_io(self):
        ret = compat.string_io("test string")
        if PY3:
            expected = "io.StringIO object"
        else:
            expected = "cStringIO.StringI object"
        self.assertTrue(expected in repr(ret))

    def test_string_io_unicode(self):
        ret = compat.string_io("test string \xf8")
        if PY3:
            expected = "io.StringIO object"
        else:
            expected = "StringIO.StringIO instance"
        self.assertTrue(expected in repr(ret))

    def test_ipv6_class__is_packed_binary(self):
        ipv6 = compat.IPv6AddressScoped("2001:db8::")
        self.assertEqual(str(ipv6), "2001:db8::")

    def test_ipv6_class__is_packed_binary_integer(self):
        ipv6 = compat.IPv6AddressScoped(42540766411282592856903984951653826560)
        self.assertEqual(str(ipv6), "2001:db8::")

    def test_ipv6_class__is_packed_binary__issue_51831(self):
        ipv6 = compat.IPv6AddressScoped(b"sixteen.digit.bn")
        self.assertEqual(str(ipv6), "7369:7874:6565:6e2e:6469:6769:742e:626e")



# -*- coding: utf-8 -*-

"""
Tests for the cron state
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging

import salt.utils.platform
from tests.support.case import ModuleCase
from tests.support.helpers import slowTest
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


@skipIf(salt.utils.platform.is_windows(), "minion is windows")
class CronTest(ModuleCase):
    """
    Validate the file state
    """

    def setUp(self):
        """
        Setup
        """
        self.run_state("user.present", name="test_cron_user")

    def tearDown(self):
        """
        Teardown
        """
        # Remove cron file
        self.run_function("cmd.run", cmd="crontab -u test_cron_user -r")

        # Delete user
        self.run_state("user.absent", name="test_cron_user")

    @slowTest
    def test_managed(self):
        """
        file.managed
        """
        ret = self.run_state(
            "cron.file", name="salt://issue-46881/cron", user="test_cron_user"
        )
        _expected = "--- \n+++ \n@@ -1 +1,2 @@\n-\n+# Lines below here are managed by Salt, do not edit\n+@hourly touch /tmp/test-file\n"
        self.assertIn(
            "changes",
            ret["cron_|-salt://issue-46881/cron_|-salt://issue-46881/cron_|-file"],
        )
        self.assertIn(
            "diff",
            ret["cron_|-salt://issue-46881/cron_|-salt://issue-46881/cron_|-file"][
                "changes"
            ],
        )
        self.assertEqual(
            _expected,
            ret["cron_|-salt://issue-46881/cron_|-salt://issue-46881/cron_|-file"][
                "changes"
            ]["diff"],
        )
